/*-------------------------------------------------------------------------------
	DSerial Simple Console
	(C) 2007 by Alexei Karpenko
-------------------------------------------------------------------------------*/

#include <nds.h>
#include <nds/arm9/console.h> //basic print funcionality
#include <stdio.h>
#include <string.h>
#include "dserial.h"
#include "firmware_bin.h"
#include "keyboard.h"

/*-------------------------------------------------------------------------------*/
void keycodeHandler(int keycode) {
/*-------------------------------------------------------------------------------*/
	if (keycode > 0) {
		char c = (char) keycode;
		iprintf("%c", c);				/* local echo */
		dseUartSendBuffer(&c, 1, true); /* blocking uart */
	}
}

/*-------------------------------------------------------------------------------*/
int main(void) {
/*-------------------------------------------------------------------------------*/

	videoSetMode(MODE_0_2D | DISPLAY_BG0_ACTIVE);
	videoSetModeSub(MODE_0_2D | DISPLAY_BG0_ACTIVE);
	vramSetBankA(VRAM_A_MAIN_BG);
	vramSetBankC(VRAM_C_SUB_BG);

	BG0_CR = BG_MAP_BASE(31);
	SUB_BG0_CR = BG_COLOR_256 | BG_MAP_BASE(31);

	BG_PALETTE[255] = RGB15(31,31,31);	//by default font will be rendered with color 255

	consoleInitDefault((u16*)SCREEN_BASE_BLOCK(31), (u16*)CHAR_BASE_BLOCK(0), 16);

	keyboardInit();
	keyboardSetKeycodeHandler(keycodeHandler);

	irqInit();
	irqSet(IRQ_VBLANK, 0);

	while(!dseInit()) {
		iprintf("DSerial is not inserted!\n");
		iprintf("Insert DSerial and press A.\n");
		
		while(~keysDown() & KEY_A) { swiWaitForVBlank(); scanKeys(); }
		while(~keysUp() & KEY_A) { swiWaitForVBlank(); scanKeys(); }
	}
	iprintf("DSerial initialized!\n");


	iprintf("\n");
	iprintf("A - Sends UART test string\n");
	iprintf("B - Reads flash\n");
	iprintf("X - Sets UART baud rate to 9600\n");
	iprintf("Y - Reads tilt data\n");
	iprintf("L - Uploads firmware to MCU\n");
	iprintf("R - Switches between FW & bootloader\n");
	iprintf("\n");

	while(1) {
		touchPosition pos;
	
		swiWaitForVBlank();

		pos = touchReadXY();
		scanKeys();
		keyboardUpdate(pos);

		if(keysDown() & KEY_A) {
			dseUartSendBuffer("Hello!", 6, true);
		} else if(keysDown() & KEY_B) {
			char buffer[32];
			dseReadFlash(buffer, 0x0800, 12);
			buffer[12] = '\0';
			iprintf(buffer);
		} else if(keysDown() & KEY_X) {
			dseUartSetBaudrate(9600); /* at boot-up, baudrate is set to 115200 bps */
		} else if(keysDown() & KEY_Y) {;
			char buffer[32];
			uint8 size = dseAdcRead(buffer);
			if (size >= 2) {
				iprintf("Tilt = (0x%02X, 0x%02X)\n", buffer[0], buffer[1]);
			}
		} else if(keysDown() & KEY_R) {
			iprintf("<Boot>");
			dseBoot();
		} else if(keysDown() & KEY_L) {
			iprintf("<Uploading Firmware>");
			dseUploadFirmware((char *) firmware_bin, firmware_bin_size);
			iprintf("<Done>");
		}
	}

	return 0;
}
